@@ -5,13 +5,11 @@ import logging
 import time
 import random
 
-def get_logger(debug=False):
-    """
+""" def get_logger(debug=False):
     Creates a logging object and returns it
 
     Returns:
         logger:logging.Log: Log object
-    """
 
     loglevel = logging.DEBUG if debug else logging.INFO
     l = logging.getLogger(__name__)
@@ -23,28 +21,11 @@ def get_logger(debug=False):
         l.addHandler(h)
         l.setLevel(loglevel)
         l.handler_set = True
-    return l  
- 
- 
-def exception(func):
-    """
-    Exception logging decorator
+    return l   """
 
-    Args:
-        func:function: Function to wrap
 
-    Returns:
-        wrapper:function: Wrapper function
-    """
-
-    @functools.wraps(func)
-    def wrapper(*args, **kwargs):
-        try:
-            return func(*args, **kwargs)
-        except:
-            # log the exception
-            msg = "Exception in method {}".format(func.__name__)
-            logger = get_logger('debug.log')
-            logger.exception(msg)
- 
-    return wrapper
+def get_url(url, scraperapi_key:str=None):
+    if scraperapi_key:
+        return f'http://api.scraperapi.com?api_key={scraperapi_key}&url={url}'
+    else:
+        return url
